pub struct GIntervalIndexMap<D> { /* private fields */ }Expand description
A Bed map that preserves the order.
Implementations
Sourceimpl<D> GIntervalIndexMap<D>
impl<D> GIntervalIndexMap<D>
pub fn new() -> Self
pub fn len(&self) -> usize
Sourcepub fn find<B: BEDLike>(
&self,
bed: &B,
) -> impl Iterator<Item = (GenomicRange, &D)>
pub fn find<B: BEDLike>( &self, bed: &B, ) -> impl Iterator<Item = (GenomicRange, &D)>
Note the results may be returned in arbitrary order.
Sourcepub fn find_index_of<B: BEDLike>(
&self,
bed: &B,
) -> GIntervalQueryIter<'_, usize> ⓘ
pub fn find_index_of<B: BEDLike>( &self, bed: &B, ) -> GIntervalQueryIter<'_, usize> ⓘ
Note the results may be returned in arbitrary order.
pub fn get(&self, index: usize) -> Option<&D>