pub struct ExternalSorter { /* private fields */ }Implementations
Sourceimpl ExternalSorter
impl ExternalSorter
Sourcepub fn sort<I, T>(
&self,
input: I,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + Ord,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
pub fn sort<I, T>(
&self,
input: I,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + Ord,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
Sourcepub fn sort_by<I, T, F>(
&self,
input: I,
cmp: F,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
F: Fn(&T, &T) -> Ordering + Sync + Send + Copy,
pub fn sort_by<I, T, F>(
&self,
input: I,
cmp: F,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
F: Fn(&T, &T) -> Ordering + Sync + Send + Copy,
Sourcepub fn sort_async<I, T>(
&self,
input: I,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + Ord + 'static,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
pub fn sort_async<I, T>(
&self,
input: I,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + Ord + 'static,
T::Archived: Deserialize<T, DiskDeserializer>,
I: IntoIterator<Item = T>,
Asynchronously sorts items using Ord and returns a sorted iterator.
This delegates to ExternalSorter::sort_by_async with T::cmp.
Errors
Returns SortError if temporary chunk creation, serialization, or
deserialization fails.
Sourcepub fn sort_by_async<I, T, F>(
&self,
input: I,
cmp: F,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
I: IntoIterator<Item = T>,
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + 'static,
T::Archived: Deserialize<T, DiskDeserializer>,
F: Fn(&T, &T) -> Ordering + Sync + Send + Copy + 'static,
pub fn sort_by_async<I, T, F>(
&self,
input: I,
cmp: F,
) -> Result<impl ExactSizeIterator<Item = Result<T, ExternalChunkError>>, SortError>where
I: IntoIterator<Item = T>,
T: Archive + for<'a> Serialize<DiskSerializer<'a>> + Send + 'static,
T::Archived: Deserialize<T, DiskDeserializer>,
F: Fn(&T, &T) -> Ordering + Sync + Send + Copy + 'static,
Asynchronously sorts items with a custom comparator and returns a sorted iterator.
Chunk sorting/spilling jobs are scheduled onto the configured rayon thread pool while input continues to be consumed on the caller thread.
Errors
Returns SortError if any background job fails while creating,
serializing, or reading temporary chunks.